from __future__ import print_function
import yaml
import unittest
import uservoice

class TestSso(unittest.TestCase):
    def setUp(self):
        super(TestSso, self).setUp()
        with open('test/config.yml') as f:
            self.config = yaml.load(f)

    def test_generating_with_validity(self):
        sso_token = uservoice.generate_sso_token(self.config['subdomain_name'], self.config['sso_key'], {
            'email': 'myemail@example.com',
            'guid': 'code-myemail@example.com-1'
        }, 10)
        print("{protocol}://{subdomain}.{domain}?sso={sso}".format(protocol=self.config.get('protocol', 'https'),domain=self.config.get('uservoice_domain', 'uservoice.com'),subdomain=self.config['subdomain_name'],sso=sso_token))
        self.assertTrue(len(sso_token) > 10)



from future import standard_library
standard_library.install_aliases()
from builtins import object
import operator
import array
import urllib.request, urllib.parse, urllib.error
import urllib.request, urllib.error, urllib.parse
import simplejson as json
import uservoice
from requests_oauthlib import OAuth1
from urllib.parse import parse_qs
import requests
version='0.0.19'

class APIError(RuntimeError): pass
class Unauthorized(APIError): pass
class NotFound(APIError): pass
class ApplicationError(APIError): pass

class Client(object):
    def __init__(self, subdomain_name, api_key, api_secret=None, oauth_token='', oauth_token_secret='', callback=None, protocol=None, uservoice_domain=None):
        self.request_token = None
        self.token = oauth_token
        self.secret = oauth_token_secret
        self.default_headers = { 'Content-Type': 'application/json', 'Accept': 'application/json',  'API-Client': 'uservoice-python-' + version }
        oauth_hooks = {}
        if api_secret:
            self.oauth = OAuth1(api_key, api_secret, resource_owner_key=self.token, resource_owner_secret=self.secret, callback_uri=callback)
        else:
            self.oauth = None
        self.api_url = "{protocol}://{subdomain_name}.{uservoice_domain}".format(
                           subdomain_name=subdomain_name,
                           protocol=(protocol or 'https'),
                           uservoice_domain=(uservoice_domain or 'uservoice.com')
                       )
        self.api_key = api_key
        self.api_secret = api_secret
        self.callback = callback
        self.subdomain_name = subdomain_name
        self.uservoice_domain = uservoice_domain
        self.protocol = protocol

    def get_request_token(self, callback=None):
        url = self.api_url + '/oauth/request_token'
        body = {}
        if self.callback or callback:
            body['oauth_callback'] = callback or self.callback
        oauth = OAuth1(self.api_key, self.api_secret, callback_uri=self.callback)
        resp = requests.post(url, body, headers=self.default_headers, auth=oauth)
        token = parse_qs(resp.text)
        if not 'oauth_token' in token or not 'oauth_token_secret' in token:
            raise Unauthorized('Failed to get request token')
        return self.login_with_access_token(token['oauth_token'][0], token['oauth_token_secret'][0])

    def authorize_url(self):
        self.request_token = self.get_request_token()
        url = self.api_url + '/oauth/authorize?oauth_token=' + self.request_token.token
        return url

    def login_with_verifier(self, verifier=None):
        url = self.api_url + '/oauth/access_token'
        oauth = OAuth1(self.api_key, self.api_secret, resource_owner_key=self.request_token.token, resource_owner_secret=self.request_token.secret, callback_uri=self.callback, verifier=verifier)
        resp = requests.post(url, auth=oauth)
        token = parse_qs(resp.text)
        return self.login_with_access_token(token['oauth_token'][0], token['oauth_token_secret'][0])

    def login_with_access_token(self, token, secret):
        return Client(self.subdomain_name, self.api_key, self.api_secret, oauth_token=token, oauth_token_secret=secret, callback=self.callback,
                                                                                                                        protocol=self.protocol,
                                                                                                                        uservoice_domain=self.uservoice_domain)

    def request(self, method, path, params={}):
        json_body = None
        get_parameters = {}
        method = method.upper()

        url = self.api_url + path
        if self.api_secret == None:
            if '?' in url:
                url += '&client=' + self.api_key
            else:
                url += '?client=' + self.api_key
        json_resp = None
        if method == 'POST':
            json_resp = requests.post(url, json.dumps(params), headers=self.default_headers, auth=self.oauth)
        elif method == 'PUT':
            json_resp = requests.put(url, json.dumps(params), headers=self.default_headers, auth=self.oauth)
        elif method == 'GET':
            json_resp = requests.get(url, headers=self.default_headers, auth=self.oauth)
        elif method == 'DELETE':
            json_resp = requests.delete(url, headers=self.default_headers, auth=self.oauth)

        attrs = {}
        try:
            if json_resp.status_code == 404:
                attrs = {'errors': {'type': 'record_not_found' }}
            else:
                attrs = json.loads(json_resp.content)
        except json.JSONDecodeError as e:
            raise APIError(e)

        if 'errors' in attrs:
            if attrs['errors']['type'] == 'unauthorized':
                raise Unauthorized(attrs)
            elif attrs['errors']['type'] == 'record_not_found':
                raise NotFound(attrs)
            elif attrs['errors']['type'] == 'application_error':
                raise ApplicationError(attrs)
            else:
                raise APIError(attrs)
        return attrs


    # handy delegate methods
    def get(self, path, params={}): return self.request('get', path, params)
    def put(self, path, params={}): return self.request('put', path, params)
    def post(self, path, params={}): return self.request('post', path, params)
    def delete(self, path, params={}): return self.request('delete', path, params)

    def get_collection(self, path, **opts):
        return uservoice.Collection(self, path, **opts)

    def login_as(self, email):
        resp = self.post('/api/v1/users/login_as', {
            'request_token': self.get_request_token().token,
            'user': { 'email': email }
        })
        if 'token' in resp:
            token = resp['token']['oauth_token']
            secret = resp['token']['oauth_token_secret']
            return self.login_with_access_token(token, secret)
        else:
            raise Unauthorized(resp)

    def login_as_owner(self):
        resp = self.post('/api/v1/users/login_as_owner', {
            'request_token': self.get_request_token().token
        })
        if 'token' in resp:
            token = resp['token']['oauth_token']
            secret = resp['token']['oauth_token_secret']
            return self.login_with_access_token(token, secret)
        else:
            raise Unauthorized(resp)

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        pass





from builtins import str
from builtins import range
from builtins import object
import yaml
import unittest
import uservoice

ELEMENTS = 1501
class TestCollection(unittest.TestCase):
    def setUp(self):
        super(TestCollection, self).setUp()
        with open('test/config.yml') as f:
            self.config = yaml.load(f)
        class MockClient(object):
            def __init__(self, result):
                self.result = result
            def get(self, path):
                return self.result
        self.clientWithEmptySet = MockClient({"response_data": {"page": 1, "per_page": uservoice.PER_PAGE, "total_records": 0, "filter": "all", "sort": "votes"}, "suggestions": []})
        self.clientWithOne = MockClient({"response_data": {"page": 1, "per_page":  uservoice.PER_PAGE, "total_records": 1, "filter": "all", "sort": "votes"},
            "suggestions": [{
                "url": "http://uservoice-subdomain.uservoice.com/forums/1-general/suggestions/1-idea",
                "id": 1,
                "state": "published",
                "title": "a",
                "text": "b",
                "formatted_text": "b",
                "forum": {"id": "1", "name": "General"}
            }]})
        class PagedClient(object):
            def get(self, path):
                page = int(path[len(path) - 1])
                page_first_index = uservoice.PER_PAGE * (page - 1) + 1
                page_last_index = min(uservoice.PER_PAGE * page, ELEMENTS)
                if "/api/v1/suggestions?per_page=" in path:
                    suggestions = []
                    for idea_index in range(page_first_index, page_last_index+1):
                        suggestions.append({
                          "url": "http://uservoice-subdomain.uservoice.com/forums/1-general/suggestions/" + str(idea_index) + "-idea",
                          "id": idea_index,
                          "state": "published",
                          "title": "Idea",
                          "text": "Idea",
                          "formatted_text": "Idea",
                          "forum": {"id": "1", "name": "General"}
                        })
                    return {
                      "response_data": {
                        "page": page,
                        "per_page": uservoice.PER_PAGE,
                        "total_records": ELEMENTS,
                        "filter": "all",
                        "sort": "votes"
                      },
                      "suggestions": suggestions
                    }
                else:
                    raise ValueError(path + 'not supported in PagedClient')
        self.pagedClient = PagedClient()

    def test_getting_zero_size(self):
        collection = uservoice.Collection(self.clientWithEmptySet, '/api/v1/suggestions')
        self.assertEqual(len(collection), 0)

    def test_enumerating_zero_elements(self):
        collection = uservoice.Collection(self.clientWithEmptySet, '/api/v1/suggestions')
        for i in collection:
            raise IndexError

    def test_getting_one_element(self):
        collection = uservoice.Collection(self.clientWithOne, '/api/v1/suggestions')
        self.assertEqual(collection[0]['id'], 1)

    def test_yielding_once(self):
        collection = uservoice.Collection(self.clientWithOne, '/api/v1/suggestions')
        count = 0
        for i in collection:
            count += 1
        self.assertEqual(count, 1)

    def test_len_of_1(self):
        collection = uservoice.Collection(self.clientWithOne, '/api/v1/suggestions')
        self.assertEqual(len(collection), 1)

    def test_correct_size_with_paged_client(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions')
        self.assertEqual(len(collection), ELEMENTS)

    def test_yield_correct_number_of_times_with_paged_client(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions')
        count = 0
        for i in collection:
            count += 1
        self.assertEqual(count, ELEMENTS)

    def test_yield_correct_number_of_times_with_paged_client_and_limit(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions', limit=1001)
        count = 0
        for i in collection:
            count += 1
        self.assertEqual(count, 1001)

    def test_limited_size_with_paged_client_and_limit(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions', limit=1001)
        self.assertEqual(len(collection), 1001)

    def test_limited_size_with_paged_client_and_limit(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions', limit=1001)
        def func():
            collection[1001]
        self.assertRaises(IndexError, func)

    def test_accessing_out_of_bounds_element_in_first_page_with_really_small_limited_size_with_paged_client(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions', limit=2)
        def func():
            collection[2]
        self.assertRaises(IndexError, func)

    def test_accessing_element_in_first_page_with_really_small_limited_size_with_paged_client(self):
        collection = uservoice.Collection(self.pagedClient, '/api/v1/suggestions', limit=2)
        self.assertEqual(collection[1]['id'], 2)




from future import standard_library
standard_library.install_aliases()
from builtins import chr
from builtins import range
from Crypto.Cipher import AES
import base64
import hashlib
import datetime
import urllib.request, urllib.parse, urllib.error
import pytz
import array
import operator
import simplejson as json
import sys

def generate_sso_token(subdomain_name, sso_key, user_attributes, valid_for = 300):
    current_time = (datetime.datetime.now(pytz.utc) + datetime.timedelta(seconds=valid_for)).strftime('%Y-%m-%d %H:%M:%S')
    user_attributes.setdefault('expires', current_time)
    user_json = json.dumps(user_attributes, separators=(',',':'))
    iv = "OpenSSL for Ruby"
    block_size = 16

    salted = sso_key + subdomain_name
    saltedHash = hashlib.sha1(salted.encode('utf8')).digest()[:16]

    json_bytes = array.array('b', user_json.encode('utf8'))
    iv_bytes = array.array('b', iv.encode('utf8'))

    for i in range(0, 16):
        json_bytes[i] = operator.xor(json_bytes[i], iv_bytes[i])

    pad = block_size - len(json_bytes) % block_size
    data = json_bytes + array.array('b', [pad] * pad)
    aes = AES.new(saltedHash, AES.MODE_CBC, iv)
    if sys.version_info[0] < 3:
        data_bytes = data.tostring()
    else:
        data_bytes = data.tobytes()
    encrypted_bytes = aes.encrypt(data_bytes)

    return urllib.parse.quote(base64.b64encode(encrypted_bytes))

